<?php

/**
 * @file
 * Entity update module (entity_update).
 *
 * This file provide drush commands tasks.
 */

use Drupal\entity_update\EntityUpdate;

/**
 * Implements hook_drush_command().
 */
function entity_update_drush_command() {
  $items = [];

  $items['entity-update'] = [
    'description' => 'Update Entity Schema',
    'aliases' => ['upe'],
    'options' => [
      'show' => dt('Show entities to update'),
      'basic' => dt('Update all entities as basic way'),
      'all' => dt('Update all Entities'),
      'nobackup' => dt('Disable automatic database backup (Not recommended)'),
      'clean' => dt('Cleanup entity backup database'),
    ],
    'examples' => [
      'drush upe --show' => 'Show entities to update.',
      'drush upe --basic' => 'Update entities. Run this command if all entities to update are empty. Else Exception.',
      'drush upe --all' => 'Update entities with data. Run this command if any entity (to update) has data.',
      'drush upe --basic --nobackup' => 'Run basic update without automatic database backup.',
      'drush upe --all --nobackup' => 'Run without automatic database backup (Not recommended).',
      'drush upe --clean' => 'Cleanup entity backup database.',
    ],
  ];

  return $items;
}

/**
 * Call back function of entity-update.
 */
function drush_entity_update() {

  // Show entities to update.
  if (drush_get_option('show')) {
    $list = EntityUpdate::getEntityTypesToUpdate();
    if (empty($list)) {
      drush_print(' -> ALl Entities are up to date');
    }
    else {
      foreach ($list as $item => $entity_type_changes) {
        drush_print(" -> $item, Change(s) : " . count($entity_type_changes));
        foreach ($entity_type_changes as $entity_change_summ) {
          drush_print("    " . strip_tags($entity_change_summ));
        }
      }
    }
    return;
  }

  // Clean entity backup database.
  if (drush_get_option('clean')) {
    EntityUpdate::cleanupEntityBackup();
    drush_print("Entity backup data removed.");
    return;
  }

  drush_print(' - If you use this module, you are conscience what you are doing. You are the responsible of your work');
  drush_print(' - Please backup your database before any action');
  drush_print(' - Please Read the documentation before any action');
  drush_print(' - Do not use this module on multi sites.');
  drush_print(' - Remove backuped data if any (Using : drush upe --clean).');

  // Backup database.
  if (!drush_get_option('nobackup')) {
    $db_backup_file = "backup_" . date("ymd-his") . ".sql.gz";
    drush_print("Backup database to : $db_backup_file");
    drush_print("To restore, run : gunzip < $db_backup_file | drush sqlc ");
    @exec('drush cr');
    @exec('drush sql-dump --gzip > ' . $db_backup_file);
  }

  // Basic entity update.
  if (drush_get_option('basic')) {
    if (drush_confirm('Are you sure you want update entities ?')) {
      $res = EntityUpdate::basicUpdate();
      drush_log('Basic entities update', $res ? 'ok' : 'error');
    }
    return;
  }

  // Update all entities.
  if (drush_get_option('all')) {
    if (drush_confirm('Are you sure you want update all entities ?')) {
      $res = EntityUpdate::safeUpdateMain();
      drush_log('End of entities update process', $res ? 'ok' : 'error');
    }
    return;
  }

  drush_log('No option specified, please type "drush help upe" for help or refer to the documentation.', 'cancel');

}
